#include "automate.h"
#include <string.h>

/*
 * Lit un automate depuis un fichier f
 */ 
Automate * automate_lecture(FILE * f)
{
    Automate * a = NULL;
    int i, etat;
    char lettre;

    // Initialisation de l'automate
    a = malloc(sizeof(Automate));
    if ( a == NULL ) {
	fprintf(stderr,"Impossible d'allouer la mémoire pour l'automate\n");
	return NULL;
    }

    a->nbetats = 0;
    a->terminaux = NULL;
    a->fleches = NULL;

    // Lecture du nombre d'état
    if ( fscanf(f, "%d", & a->nbetats) != 1 ) {
	perror("fscanf");
	free(a);
	return NULL;
    }

    // On passe le retour chariot
    if ( getc(f) != '\n' ) {
	fprintf(stderr,"Automate mal défini\n");
	free(a);
	return NULL;
    }

    // Allocation de mémoire
    a->terminaux = malloc(sizeof(bool) * a->nbetats);
    if ( a->terminaux == NULL ) {
	fprintf(stderr,
		"Impossible d'allouer la mémoire pour stocker les terminaux\n");
	free(a);
	return NULL;
    }

    a->fleches = calloc(a->nbetats, sizeof(liste));
    if ( a->fleches == NULL ) {
	fprintf(stderr,
		"Impossible d'allouer la mmoire pour stocker les flches\n");
	free(a->terminaux);
	free(a);
	return NULL;
    }

    // Lecture de l'automate
    for ( i=0 ; i < a->nbetats ; i++ ) {
	// Lecture de la "terminalité"
	switch(getc(f)) {
	    case '+':
		a->terminaux[i] = VRAI;
		break;
	    case '-':
		a->terminaux[i] = FAUX;
		break;
	    default:
		goto error;
	}
	
	// Lecture des flèches
	while ( (lettre = getc(f)) != '\n' ) {
	    // Lettre
	    if ( lettre == EOF )
		goto error;
	    if ( getc(f) != ',' )
		goto error;

	    if ( liste_presence(a->fleches[i],lettre) )
		goto error;

	    // État
	    if ( fscanf(f,"%d",&etat) != 1 )
		goto error;
	    if ( getc(f) != ';' )
		goto error;

	    // Ajout
	    if ( ! liste_ajout(&(a->fleches[i]), etat, lettre) ) {
		fprintf(stderr,"Allocation mémoire impossible\n");
		goto error_nowarn;
	    }
	}
    }

    return a;

error:
    fprintf(stderr,"Automate mal défini\n");
error_nowarn:
    free(a->terminaux);
    for ( i=0 ; i < a->nbetats ; i++ )
	liste_detruire(a->fleches[i]);
    free(a->fleches);
    free(a);

    return NULL;
}

/*
 * crit l'automate dans le fichier f
 */
static int automate_print(Automate * a, FILE * f)
{
    int i; liste l;

    fprintf(f,"%d\n",a->nbetats);
    
    for ( i=0 ; i < a->nbetats ; i++ ) {
	// "Terminalité"
	if ( a->terminaux[i] ) {
	    if ( fputc('+',f) == EOF ) {
		perror("fputc");
		return 0;
	    }
	} else {
	    if ( fputc('-',f) == EOF ) {
		perror("fputc");
		return 0;
	    }
	}

	// Couples
	l = a->fleches[i];
	while ( l != NULL ) {
	    fprintf(f,"%c,%d;",l->lettre,l->etat);
	    l = l->suiv;
	}
	if ( fputc('\n',f) == EOF ) {
	    perror("fputc");
	    return 0;
	}
    }

    return 1;
}

/*
 * Affiche l'automate donné
 */
inline void automate_affiche(Automate * a)
{
    if ( ! automate_print(a,stdout) ) {
	fprintf(stderr,"Impossible d'afficher l'automate !");
    }
}

/*
 * Enregistre l'automate donné dans un fichier
 */
int automate_enregistre(Automate * a, char * fichier)
{
    assert(fichier);

    FILE * f = NULL;
    if ( (f = fopen(fichier,"w")) == NULL ) {
	perror("fopen");
	return 0;
    }

    automate_print(a,f);

    fclose(f);

    return 1;
}

/*
 * Libère la mémoire occupée par l'automate a
 */
void automate_detruire(Automate * a)
{
    int i;

    free(a->terminaux);
    for ( i=0 ; i < a->nbetats ; i++ )
	    liste_detruire(a->fleches[i]);
    free(a->fleches);   
    
    free(a);
}

void automate_sort(Automate *a)
{
    int i;
    for ( i=0 ; i < a->nbetats ; i++ )
	liste_sort(&(a->fleches[i]));
}

/*
 * Copie in dans out
 */
int automate_copie(Automate ** out, Automate * in)
{
    int i;
    liste tmp;

    *out = malloc(sizeof(Automate));
    if ( *out == NULL )
	return 0;

    (*out)->nbetats = in->nbetats;

    if ( ((*out)->terminaux = malloc((*out)->nbetats * sizeof(bool))) == NULL )
	return 0;
    if ( ((*out)->fleches = calloc((*out)->nbetats,sizeof(liste))) == NULL )
	return 0;
    
    for ( i=0 ; i < (*out)->nbetats ; i++ )
	(*out)->terminaux[i] = in->terminaux[i];

    for ( i=0 ; i < (*out)->nbetats ; i++ ) {
	tmp = in->fleches[i];
	while ( tmp != NULL ) {
	    if ( ! liste_ajout(&((*out)->fleches[i]), tmp->etat, tmp->lettre) )
		return 0;
	    tmp = tmp->suiv;
	}
    }

    return 1;
}

