#include "langue.h"

/*
 * Libre la mmoire associe  la langue
 */
void langue_libere(Langue * l)
{
    int i;
    for ( i=0 ; i < l->nb ; i++ )
	free(l->mots[i]);
    free(l->mots);
    free(l);
}

/*
 * Ajoute un mot  la langue
 */
int langue_ajoute(Langue * l, char * mot)
{
    assert(l);

    l->mots = realloc(l->mots, (l->nb+1)*sizeof(char **));
    if ( l->mots == NULL )
	return 0;

    l->mots[l->nb] = malloc(strlen(mot)+1);
    if ( l->mots[l->nb] == NULL )
	return 0;

    strcpy(l->mots[l->nb], mot);
    
    l->nb++;

    return 1;
}

// Fonction de comparaison pour qsort()
static int compar(const void * mot1, const void * mot2)
{
    int a,b;
    if ( (a = strlen(*(char **)mot1)) != (b = strlen(*(char **)mot2)) )
	return a-b;
    return strcmp(*(char **)mot1, *(char **)mot2);
}

/*
 * Met dans l'ordre et efface les doublons
 * ordre lexicographique ET ordre de longueur
 */
static void unbordelize(Langue * l)
{
    int i, c = 0;

    qsort(l->mots, l->nb, sizeof(char *), compar);

    // Efface les doublons
    if ( l->nb > 0 )
	for ( i=1 ; i < l->nb ; i++ ) {
	    if ( strcmp(l->mots[c],l->mots[i]) == 0 ) {
		free(l->mots[i]);
		l->mots[i] = NULL;
	    } else
		c = i;
	}
}

/*
 * Cherche les mots du langage de l'automate
 */
static int etcestpartit(char *mot,int taillemot,Automate * a, Langue * l, int etat, int pos)
{
    
    if ( etat == a->nbetats-1 )
	return 1;

    liste fleches = a->fleches[etat];
    
    // Gestion mmoire
    if ( pos == taillemot ) {
	taillemot += 42;
	mot = realloc(mot,taillemot);
	if ( mot == NULL )
	    return 0;
    }
	
    // Dtection de mot
    if ( a->terminaux[etat] )
    {
	if(etat == 0)
	{
		mot[0]=' ';
		mot[1]='\0';
	}
	mot[pos]='\0';
	if ( ! langue_ajoute(l, mot) )
	    return 0;
    }

    // Prochains mots
    while ( fleches != NULL ) {
	mot[pos] = fleches->lettre;
	if ( fleches->etat != etat ) // vite les mots "infinis"
	    if ( ! etcestpartit(mot,taillemot,a,l,fleches->etat,pos+1) )
		return 0;
	fleches = fleches->suiv;
    }
    
    return 1;
}

/*
 * Calcule TOUS les mots engendrs par l'automate
 */
Langue * langue_automate(Automate * a)
{
    Langue * l = NULL;

    char *mot=NULL;
    int taillemot=0;
    l =(Langue*) malloc(sizeof(Langue));
    if ( l == NULL )
	return NULL;

    l->mots = NULL;
    l->nb = 0;

    if ( ! etcestpartit(mot,taillemot,a,l,0,0) ) {
	langue_libere(l);
	return NULL;
    }

    unbordelize(l);
    
    return l;
}

/*
 * Retourne une nouvelle langue contenant les plus petits mots de l
 * NOTE : La langue doit tre au pralable "unbordelize"
 */
Langue * langue_mini(Langue * l)
{
    Langue * mini = NULL;
    int i;
    
    mini = calloc(1, sizeof(Langue));
    if ( mini == NULL )
	return NULL;

    for ( i=0 ; i < l->nb ; i++ ) {
	if ( i > 0 && strlen(l->mots[i]) != strlen(l->mots[i-1]) )
	    break;
	langue_ajoute(mini, l->mots[i]);
    }

    return mini;
}

/*
 * Affiche la langue l dans f
 */
int langue_print(Langue * l, FILE * f)
{
    int i;
    for ( i=0 ; i < l->nb ; i++ ) {
	if ( l->mots[i] != NULL ) {
	    if ( fputs(l->mots[i],f) < 0 ) {
		perror("fputs");
		return 0;
	    }
	    if ( fputc('\n',f) == EOF ) {
		perror("fputc");
		return 0;
	    }
	}
    }

    return 1;
}

/* 
 * Affiche la langue l
 */
inline void langue_affiche(Langue * l)
{
    langue_print(l,stdout);
}

/*
 * Enregistre la langue l dans un fichier
 */
int langue_enregistre(Langue * l, char * fichier)
{
    FILE * plop = NULL;

    plop = fopen(fichier,"w");
    if ( plop == NULL ) {
	perror("fopen");
	return 0;
    }

    if ( ! langue_print(l,plop) )
	return 0;

    fclose(plop);

    return 1;
}

// Calcule la liste d mots qui differre entre les deux listes
Langue* langue_diff(Langue* l1, Langue *l2)
{
    Langue *l=NULL;
    int i=0,stop=0,j=0;
    int nb=0;
    int temp=0,p=1000;
    
    unbordelize(l1);
    unbordelize(l2);
    
    l=calloc(1,sizeof(Langue));

    for(i=0;i<l1->nb;i++)
    {
	    stop=1;
	    for(j=0;j<l2->nb;j++)
	    {
		    if(strcmp(l1->mots[i],l2->mots[j])==0)
		    {
			    stop=0;
			    break;
		    }
	    }
	    if(stop == 1)
	    {
		langue_ajoute(l, l1->mots[i]);
		nb++;
	    }
    }
    for(i=0;i<l2->nb;i++)
    {
	    stop=1;
	    for(j=0;j<l1->nb;j++)
	    {
		    if(strcmp(l2->mots[i],l1->mots[j])==0)
		    {
			    stop=0;
			    break;
		    }
	    }
	    if(stop == 1)
	    {
		langue_ajoute(l, l2->mots[i]);
		nb++;
	    }
    }

    l->nb=nb;
    unbordelize(l);
    for(i=0;i<l->nb;i++)
    {
	    if(l->mots[i] != NULL)
		temp=strlen(l->mots[i]);
	    if(temp <= p)
		p=temp;
	    else
	    {
		free(l->mots[i]);
		l->mots[i]=NULL;
	    }
    }
    return l;
}
