#include "sousmot.h"
#include "langue.h"


/* Determine l'alphabet qui constitue le mot
 * */
static char* alphabet(char* mot, int *taille)
{
    char *a=NULL;
    int i=0,pos=1,j=0;
    *taille=0;
       
    a=(char*)malloc(sizeof(char)*256);
    a[0]=mot[0];
    for(i=0;i<strlen(mot)+1;i++)
    {
	for(j=0;j<pos;j++)
	{
	    if(a[j] == mot[i])
	        break;
	}
	if(j == pos)
	{
	    a[pos]=mot[i];
	    *taille+=1;
	    pos++;
	}
    }
    return a;
}

/* Automate des sousmots
* */
Automate* sousmot_automate(char* mot)
{
    int i=0,pos=0,j=0,stop=0;
    int saut=0;
    Automate* a = NULL;
    char *alpha=NULL;
    liste l,del,res=NULL;
    int taille;
    int *etat=NULL;
    if((a=(Automate*)malloc(sizeof(Automate))) == NULL)
    {
	fprintf(stderr,"Erreur d'allocation dans automate_sousmot");
	return NULL;
    }

    a->nbetats=strlen(mot)+2;
    a->terminaux=NULL;
    a->fleches=NULL;
    
    if((a->terminaux=(bool*)malloc(sizeof(bool)*a->nbetats)) == NULL)
    {
	free(a);
	return NULL;
    }
    // allocation des fleches
    if((a->fleches=(liste*)calloc(a->nbetats,sizeof(liste))) == NULL)
    {
	fprintf(stderr,"impossible d'allouer les fleches");
	free(a->terminaux);
	free(a);
	return NULL;
    }
    
    // etats terminaux
    for(i=0;i<a->nbetats-1;i++) 
	    a->terminaux[i]=VRAI;
    // etat poubelle
    a->terminaux[i]=FAUX;

    
    for(i=1;i<(a->nbetats-1);i++)
	if( ! liste_ajout(&(a->fleches[i-1]),i,mot[i-1]))
	{
	    fprintf(stderr,"Allocation memoire impossible");
	    automate_detruire(a);
	    return NULL;
	}
    
    for(saut=2;saut<a->nbetats-1;saut++)
    {
    	for(i=0;i+saut<a->nbetats-1;i++)
	{
		liste_ajout(&(a->fleches[i]),i+saut,mot[i+saut-1]);
	}
    }
    
    // on determine le nombre de fleches pour chaque etats
    alpha=alphabet(mot,&taille);
   
    // on supprime les fleches en trop
    for(i=0;i<a->nbetats;i++)
    {	
	l=a->fleches[i];
	del=l;
	while(l!=NULL)
	{
	    l=l->suiv;
	    if(liste_presence(l,del->lettre))
	    {
		free(del);
		del=l;
	    }
	    else
	    {
		    del->suiv=res;
		    res=del;
		    del=l;
	    }
	}
	a->fleches[i]=res;
	res=NULL;
	    
    }
    
    etat=(int*)calloc(a->nbetats,sizeof(int));
    etat[j]=0;
    j++;
    automate_sort(a);

    // on note l'ordre de parcours des etats
    for(i=0;i<a->nbetats;i++)
    {
	l=a->fleches[i];
	while(l!=NULL)
	{
	    stop=1;
	    for(pos=0;pos<j;pos++)
	    {
		if(l->etat == etat[pos])
			stop=0;
	    }
	    if(stop == 1)
	    {
		etat[j]=l->etat;
		j++;
	    }
	    l=l->suiv;
	}
    }
    // On complete l'automate
    for(i=0;i<a->nbetats;i++)
    {
	pos=0;
	l=a->fleches[i];
	while(alpha[pos]!='\0')
	{
	    if(liste_presence(l,alpha[pos]) == 0)
		liste_ajout(&(a->fleches[i]),a->nbetats-1,alpha[pos]);
    	    pos++;
	}
    }

    etat[a->nbetats-1]=a->nbetats-1;
    // on numerote les etats selon un ordre de parcours en largeur
    for(j=0;j<a->nbetats-1;j++)
    {
    	    l=a->fleches[j];

	    while(l!=NULL)
	    {
		for(i=0;i<a->nbetats;i++)
			if(l->etat==etat[i])
			{
				l->etat=i;
				break;
			}
    		l=l->suiv;
	    }
    }

    for(j=0;j<a->nbetats-1;j++)
    {
	if(etat[j]>j)
	{
	    l=a->fleches[j];
	    a->fleches[j]=a->fleches[etat[j]];
	    a->fleches[etat[j]]=l;
	}
    }
    free(etat);
    free(alpha);
    return a;
}

/*
 * Automate des sous-mots plus petit qui differe entre deux mots
 */
Automate* sousmot_plusPetit(Automate* a, Automate* b)
{
    Automate *c;
    int i=0;
    liste l=NULL;
    liste l1=NULL,l2=NULL;
    liste temp=NULL,new=NULL;
    int nbetat[250];
    int pos=0;
    int chemin=0;
    char lettre;
    
    for(i=0;i<a->nbetats;i++)
    {
	l=a->fleches[i];
	new=NULL;
	while(l!=NULL)
	{
	    if(l->etat == a->nbetats-1)
	    {
		temp=l;
		l=l->suiv;
		free(temp);
	    }
	    else
	    {
    		liste_ajout(&(new),l->etat,l->lettre);
		temp=l;
		l=l->suiv;
		free(temp);
	    }

	}
    	if(l==NULL)
	   a->fleches[i]=NULL;
	if(new!=NULL)
	    a->fleches[i]=new;
		
    }
    for(i=0;i<b->nbetats;i++)
    {
	l=b->fleches[i];
	new=NULL;
	while(l!=NULL)
	{
	    if(l->etat == b->nbetats-1)
	    {
		temp=l;
		l=l->suiv;
		free(temp);
	    }
	    else
	    {
		liste_ajout(&(new),l->etat,l->lettre);
		temp=l;
		l=l->suiv;
		free(temp);
	    }

	}
	if(l==NULL)
	    b->fleches[i]=NULL;
	if(new!=NULL)
	    b->fleches[i]=new;
		
    }
    
    for(i=0;i<a->nbetats-1;i++)
    {
	    l=a->fleches[i];
	    while(l!=NULL)
	    {
		temp=NULL;  
		if(i<b->nbetats)
		    if(liste_presence(b->fleches[i],l->lettre)==NULL)
		    {
			    fprintf(stderr,"%d %c %d\n",l->etat,l->lettre,i);
			    nbetat[pos]=i;
			    pos++;
		    }
		l=l->suiv;
	    }
    }

    // automate des plus petits sous-mots
    if((c=(Automate*)malloc(sizeof(Automate)))==NULL)
	    return NULL;
    c->nbetats=nbetat[0]+2;
    // allocation des fleches
    c->fleches=calloc(c->nbetats,sizeof(liste*));
    // les terminaux
    c->terminaux=(bool*)malloc(sizeof(bool)*c->nbetats);
    for(i=0;i<nbetat[0]-2;i++)
	c->terminaux[i]=FAUX;
    c->terminaux[i]=VRAI;
    c->terminaux[i+1]=FAUX;

    while(i<=nbetat[0]-1)
    {
	l1=NULL;
	l2=NULL;
	if(i<a->nbetats)
	{
	    l1=a->fleches[i];
	}
	if(i<b->nbetats)
	{
	    l2=b->fleches[i];
	}
	while(l1!=NULL)
	{
	    if((l1->etat>chemin)&&(l1->etat<=nbetat[0]))
	    {
		chemin=l1->etat;
		lettre=l1->lettre;
	    }
	    l1=l1->suiv;
	}
	while(l2!=NULL)
	{
	    if((l2->etat>chemin)&&(l2->etat<=nbetat[0]))
	    {
		chemin=l2->etat;
		lettre=l2->lettre;
	    }
	    l2=l2->suiv;
	}
	liste_ajout(&(c->fleches[i]),chemin,lettre);
	i=chemin;
    }
    

    return c;
}

// Automate des plus petits sous-mots qui differencie mot et mot2
Automate* sousmot_listeDiff(char* mot1, char *mot2)
{
    Automate *a=NULL,*b=NULL,*c=NULL;
    char *a1=NULL,*a2=NULL;
    int  t1=0,t2=0;
    int  i=0,j=0,k=0;
    int  in=0;
    char *fleches;
   // Langue* l=NULL;

    // Les mots sont identiques
    if(!strcmp(mot1,mot2))
    {
	fprintf(stderr,"Les mots sont identiques\n");
	return NULL;
    }


    // Les alphabets sont differents
    a1=alphabet(mot1,&t1);
    a2=alphabet(mot2,&t2);

    if(t1>t2)
	fleches=(char*)malloc(sizeof(char)*t1);
    else
	fleches=(char*)malloc(sizeof(char)*t2);
    
    in=1;    
    while(i<t1)
    {
	for(j=0;j<t2;j++)
	{
	    in=0;
	    if(mot1[i] == mot2[j])
	    {
		in=1;
		break;
	    }
	}
	if(in == 0)
	{
	    fleches[k]=mot1[i];
	    k++;
	}
	i++;
    }
    i=0;
    in=1;
    while(i<t2)
    {
	for(j=0;j<t1;j++)
	{
	    in=0;
	    if(mot2[i] == mot1[j])
	    {
		in=1;
		break;
	    }
	}
	if(in == 0)
	{
	    fleches[k]=mot2[i];
	    k++;
	}
	i++;
    }
    
    // Les alphabets sont differents construction de l'automate
    if(k != 0)
    {
	// Creation de l'automate des plus petit sous-mots
	if((c=(Automate*)malloc(sizeof(Automate)))==NULL)
	{
	    printf("Erreur pour allouer l'automate dans sousmot_listeDiff\n");
	    return NULL;
	}
	c->nbetats=3;
	c->fleches=calloc(c->nbetats,sizeof(liste*));
	for(i=0;i<c->nbetats;i++)
		for(j=0;j<k;j++)
		    if(i == c->nbetats-1)
			liste_ajout(&(c->fleches[i]),i,fleches[j]);
		    else
			liste_ajout(&(c->fleches[i]),i+1,fleches[j]);
	c->terminaux=(bool*)malloc(sizeof(bool)*c->nbetats);
	c->terminaux[0]=FAUX;
	c->terminaux[1]=VRAI;
	c->terminaux[2]=FAUX;

	free(a1);
	free(a2);
	return c;
    }
    
    // Mots de meme lettres dans des ordres differents
    
    a=sousmot_automate(mot1);
    b=sousmot_automate(mot2);
    
    automate_sort(a);
    automate_sort(b);

    // automate produit
    sousmot_plusPetit(a, b);
    
      
 
    
    automate_detruire(a);
    automate_detruire(b);

    free(a1);
    free(a2);
    return c;
}


