#include "supermenu.h"

/*
 * Aid du "NCURSES Programming HOWTO"
 */

/*
 * Efface la ligne affichant l'aide du menu
 */
static void clearhelp(WINDOW * win)
{
    char line[74];
    memset(line, ' ', 73);
    line[73] = 0;
    mvwprintw(win,6,4,line);
}

/*
 * Initialisation de Ncurses et toute la dco
 */
static void initcurses(void)
{
    initscr();
    cbreak();
    noecho();
    keypad(stdscr, TRUE);

    if ( has_colors() ) {
	start_color();
	init_pair(1, COLOR_BLUE, COLOR_BLACK);
    }
}

/*
 * Titre et ligne horizontale
 */
static void draw_title(void)
{
    mvprintw(0, 0, "Sous-mots");
    mvhline(1, 1, ACS_HLINE, 78);
    refresh();
}

/*
 * Affiche un menu de slection
 * Retourne l'objet slectionner
 * 1 : Automate des sous-mots ( -u )
 * 2 : quivalence d'automates ( -e et -E )
 * 3 : Automate des plus petits sous-mots ( -m et -a )
 * 4 : Quitter
 */
static Action selection(void)
{
    int res = 0;
    int i = 0;
    ITEM * items[5];
    MENU * menu;
    WINDOW * win;

    // Le curseur, c'est moche.
    curs_set(0);

    // Vient vient mon joli menu
    items[0] = new_item("Sous-mot", "");
    items[1] = new_item("Equivalence", "");
    items[2] = new_item("Plus petits sous-mots", "");
    items[3] = new_item("Quitter", "");
    items[4] = NULL;
    menu = new_menu(items);

    // C'est dans la boite
    win = newwin(10, 80, 2, 0);
    keypad(win, TRUE);
    set_menu_win(menu, win);
    set_menu_sub(menu, derwin(win, 5, 78, 1, 1));

    // Parce que je le vaut bien (c), (R), TM
    set_menu_mark(menu, "> ");

    draw_title();

    // Picasso
    post_menu(menu);
    wrefresh(win);
    
    // starthelp()
    clearhelp(win);
    mvwprintw(win,6,5,
	    "Calcule l'automate des sous-mots d'un mot donn");

    // Et tu getch, et tu getch, et tu getch
    while ( (i = wgetch(win)) != '\n' )
    {
	switch(i) {
	    // Lofteur up !
	    case KEY_UP:
		if ( menu_driver(menu, REQ_UP_ITEM) == E_OK )
		    res--;
		break;
	    // Lofteur down !
	    case KEY_DOWN:
		if ( menu_driver(menu, REQ_DOWN_ITEM) == E_OK )
		    res++;
		break;
	}
	// Help me
	switch(res) {
	    case 0:
		clearhelp(win);
		mvwprintw(win,6,5,
			"Calcule l'automate des sous-mots d'un mot donn.");
		break;
	    case 1:
		clearhelp(win);
		mvwprintw(win,6,5,"Teste l'quivalence de deux automates.");
		break;
	    case 2:
		clearhelp(win);
		mvwprintw(win,6,5,"Calcule l'automate des plus petits "
			"sous-mots sparant deux mots donns.");
		break;
	    case 3:
		clearhelp(win);
		mvwprintw(win,6,5,"Quitte.");
		break;
	}
	refresh();
	wrefresh(win);
    }

    // wc() encore une victoire de canard !
    unpost_menu(menu);
    free_menu(menu);
    for ( i = 0 ; i < 4 ; i++ )
	free_item(items[i]);
    delwin(win);

    return res;
}

/*
 * Affiche une demande mot pour ensuite affiche l'automate des sous-mots
 * correspondant.
 */
static void sousmots(void)
{
    char mot[72], filename[72], * execute = NULL; 
    int posmot = 0, posfile = 0;
    Automate * res = NULL;

    filename[0] = mot[0] = 0;

    FIELD * fields[3];
    FORM * form;
    int i, curfield = 0;

    curs_set(1);

    fields[0] = new_field(1, 71, 5, 4, 0, 0);
    fields[1] = new_field(1, 71, 8, 4, 0, 0);
    fields[2] = NULL;

    set_field_back(fields[0], A_UNDERLINE);
    field_opts_off(fields[0], O_AUTOSKIP);
    set_field_back(fields[1], A_UNDERLINE);
    field_opts_off(fields[1], O_AUTOSKIP);

    form = new_form(fields);
    post_form(form);
    refresh();

    mvprintw(3, 1, "> Automate des sous-mots");
    mvprintw(4, 4, "> Mot  traiter :");
    mvprintw(7, 4, "> Fichier pour enregistrer le rsultat : "
	    "(laisser vide pour aucun)");
    refresh();

    draw_title();

    // Place le curseur sur le premier champ
    form_driver(form, REQ_FIRST_FIELD);

    while (1) {
	i = getch();
	if ( i == '\n' && curfield ) // Sortie
	    break;
	switch(i) {
	// Effacer un caractre
	    case KEY_BACKSPACE:
		form_driver(form, REQ_LEFT_CHAR);
		form_driver(form, REQ_DEL_CHAR);
		if ( curfield == 0 ) {
		    mot[posmot] = 0; 
		    posmot = (posmot != 0) ? posmot-1 : 0;
		} else {
		    filename[posfile] = 0;
		    posfile = (posfile != 0) ? posfile-1 : 0;
		}
		break;
	// Changement de champ
	    case KEY_UP:
		curfield = !curfield;
		form_driver(form, REQ_PREV_FIELD);
		form_driver(form, REQ_END_LINE);
		break;
	    case '\n':
	    case KEY_DOWN:
	    case 9: // Tabulation
		curfield = !curfield;
		form_driver(form, REQ_NEXT_FIELD);
		form_driver(form, REQ_END_LINE);
		break;
	// Remplissage de champ
	    default:
		form_driver(form, i);
		if ( curfield == 0 ) {
		    mot[posmot] = i;
		    posmot = (posmot != 72) ? posmot+1 : 71;
		} else {
		    filename[posfile] = i;
		    posfile = (posfile != 72) ? posfile+1 : 71;
		}
		break;
	}
    }

    mot[posmot] = 0;
    filename[posfile] = 0;

    unpost_form(form);
    free_form(form);
    free_field(fields[0]);

    // Quitte ncurses temporairement
    def_prog_mode();
    endwin();

    // Bourreau, fait ton office !
    res = sousmot_automate(mot);
    if ( res == NULL )
	fprintf(stderr,"Impossible de crer l'automate des sous-mots de %s...",
		mot);
    else {
	if (filename[0] == 0) {
	    automate_enregistre(res, "/tmp/plop");
	    system("less /tmp/plop");
	    unlink("/tmp/plop");
	} else {
	    automate_enregistre(res, filename);
	    execute = malloc(strlen(filename)+6);
	    strcpy(execute, "less ");
	    strcat(execute, filename);
	    system(execute);
	}
    }

    // Ncurses : le retour.
    reset_prog_mode();
    refresh();
}

/*
 * Affiche une demande de deux fichiers pour tester l'quivalence des
 * automates qu'ils contiennent.
 */
static void equiv(void)
{
    char file1[72], file2[72], diff = 0;
    int posfile1 = 0, posfile2 = 0;

    FIELD * fields[4];
    FORM * form;
    int i, curfield = 0;

    file1[0] = 0;
    file2[0] = 0;

    curs_set(1);

    fields[0] = new_field(1, 71, 5, 4, 0, 0);
    fields[1] = new_field(1, 71, 8, 4, 0, 0);
    fields[2] = new_field(1, 1, 11, 4, 0, 0);
    fields[3] = NULL;

    set_field_back(fields[0], A_UNDERLINE);
    field_opts_off(fields[0], O_AUTOSKIP);
    set_field_back(fields[1], A_UNDERLINE);
    field_opts_off(fields[1], O_AUTOSKIP);
    set_field_back(fields[2], A_UNDERLINE);
    field_opts_off(fields[2], O_AUTOSKIP);

    form = new_form(fields);
    post_form(form);
    refresh();

    mvprintw(3, 1, "> quivalence entre automates");
    mvprintw(4, 4, "> Fichier du premier automate : ");
    mvprintw(7, 4, "> Fichier du second automate : ");
    mvprintw(10,4, "> Afficher un mot qui les diffrentie ? : [o/n]");
    refresh();

    draw_title();

    // Place le curseur sur le premier champ
    form_driver(form, REQ_FIRST_FIELD);

    while (1) {
	i = getch();
	if ( i == '\n' && curfield == 2 ) // Par ici la sortie !
	    break;
	switch(i) {
	// Effacer un caractre
	    case KEY_BACKSPACE:
		form_driver(form, REQ_LEFT_CHAR); // Recule
		form_driver(form, REQ_DEL_CHAR); // Efface
		if ( curfield == 0 ) {
		    file1[posfile1] = 0; 
		    posfile1 = (posfile1 != 0) ? posfile1-1 : 0;
		} else if ( curfield == 1 ) {
		    file2[posfile2] = 0; 
		    posfile2 = (posfile2 != 0) ? posfile2-1 : 0;
		}
		break;
	// Changement de champ
	    case KEY_UP:
		curfield = (curfield - 1)%3;
		form_driver(form, REQ_PREV_FIELD);
		form_driver(form, REQ_END_LINE);
		break;
	    case '\n':
	    case KEY_DOWN:
	    case 9: // Tabulation
		curfield = (curfield + 1)%3;
		form_driver(form, REQ_NEXT_FIELD);
		form_driver(form, REQ_END_LINE);
		break;
	// Remplissage de champ
	    default:
		form_driver(form, i);
		if ( curfield == 0 ) {
		    file1[posfile1] = i;
		    posfile1 = (posfile1 != 72) ? posfile1+1 : 71;
		} else if ( curfield == 1 ) {
		    file2[posfile2] = i;
		    posfile2 = (posfile2 != 72) ? posfile2+1 : 71;
		} else {
		    if ( i == 'o' )
			diff = 1;
		    else
			diff = 0;
		}
		break;
	}
    }

    file1[posfile1] = 0;
    file2[posfile2] = 0;

    unpost_form(form);
    free_form(form);
    free_field(fields[0]);

    // Quitte ncurses temporairement
    def_prog_mode();
    endwin();

    // xcution !
    if ( equivalence_automate(file1,file2,diff) == 0 )
	printf("%s et %s sont quivalents\n",file1,file2);
    else
	printf("%s et %s ne sont pas quivalents\n",file1,file2);
    fflush(stdout);
    system("read -n 1"); // **kof**
    
    // Ncurses : le retour.
    reset_prog_mode();
    refresh();
}

/*
 * Demande 2 mots pour afficher l'automate des sous-mots sparant 
 * ces 2 mots.
 */
static void ppsm(void)
{
    char mot1[72], mot2[72], fichier[72];
    int posmot1 = 0, posmot2 = 0, posfichier = 0;
    char * execute;

    FIELD * fields[4];
    FORM * form;
    int i, curfield = 0;

    mot1[0] = 0;
    mot2[0] = 0;
    fichier[0] = 0;

    curs_set(1);

    fields[0] = new_field(1, 71, 5, 4, 0, 0);
    fields[1] = new_field(1, 71, 8, 4, 0, 0);
    fields[2] = new_field(1, 71, 11, 4, 0, 0);
    fields[3] = NULL;

    set_field_back(fields[0], A_UNDERLINE);
    field_opts_off(fields[0], O_AUTOSKIP);
    set_field_back(fields[1], A_UNDERLINE);
    field_opts_off(fields[1], O_AUTOSKIP);
    set_field_back(fields[2], A_UNDERLINE);
    field_opts_off(fields[2], O_AUTOSKIP);

    form = new_form(fields);
    post_form(form);
    refresh();

    mvprintw(3, 1, "> Liste de sous-mots sparant deux mots");
    mvprintw(4, 4, "> Premier mot : ");
    mvprintw(7, 4, "> Second mot : ");
    mvprintw(10,4, "> Fichier pour enregistrer le rsultat : "
	    "(laisser vide pour aucun)");
    refresh();

    draw_title();

    // Place le curseur sur le premier champ
    form_driver(form, REQ_FIRST_FIELD);

    while (1) {
	i = getch();
	if ( i == '\n' && curfield == 2 ) // Par ici la sortie !
	    break;
	switch(i) {
	// Effacer un caractre
	    case KEY_BACKSPACE:
		form_driver(form, REQ_LEFT_CHAR); // Recule
		form_driver(form, REQ_DEL_CHAR); // Efface
		if ( curfield == 0 ) {
		    mot1[posmot1] = 0; 
		    posmot1 = (posmot1 != 0) ? posmot1-1 : 0;
		} else if ( curfield == 1 ) {
		    mot2[posmot2] = 0; 
		    posmot2 = (posmot2 != 0) ? posmot2-1 : 0;
		} else {
		    fichier[posfichier] = 0;
		    posfichier = (posfichier != 0) ? posfichier-1 : 0;
		}
		break;
	// Changement de champ
	    case KEY_UP:
		curfield = (curfield - 1)%3;
		form_driver(form, REQ_PREV_FIELD);
		form_driver(form, REQ_END_LINE);
		break;
	    case '\n':
	    case KEY_DOWN:
	    case 9: // Tabulation
		curfield = (curfield + 1)%3;
		form_driver(form, REQ_NEXT_FIELD);
		form_driver(form, REQ_END_LINE);
		break;
	// Remplissage de champ
	    default:
		form_driver(form, i);
		if ( curfield == 0 ) {
		    mot1[posmot1] = i;
		    posmot1 = (posmot1 != 72) ? posmot1+1 : 71;
		} else if ( curfield == 1 ) {
		    mot2[posmot2] = i;
		    posmot2 = (posmot2 != 72) ? posmot2+1 : 71;
		} else {
		    fichier[posfichier] = i;
		    posfichier = (posfichier != 72) ? posfichier+1 : 71;
		}
		break;
	}
    }

    mot1[posmot1] = 0;
    mot2[posmot2] = 0;
    fichier[posfichier] = 0;

    unpost_form(form);
    free_form(form);
    free_field(fields[0]);

    // Quitte ncurses temporairement
    def_prog_mode();
    endwin();

    // xcution !
    Automate * s1 = sousmot_automate(mot1);
    Automate * s2 = sousmot_automate(mot2);
    Langue * l1 = langue_automate(s1);
    Langue * l2 = langue_automate(s2);
    Langue * diff = langue_diff(l1,l2);
    if ( fichier[0] == 0 ) {
	langue_affiche(diff);
    } else {
	langue_enregistre(diff,fichier);
	execute = malloc(strlen(fichier)+6);
	strcpy(execute,"less ");
	strcat(execute,fichier);
	system(execute);
	free(execute);
    }
    automate_detruire(s1); 
    automate_detruire(s2);
    langue_libere(l1);
    langue_libere(l2);
    langue_libere(diff);
       
    // Ncurses : le retour.
    reset_prog_mode();
    refresh();
}

void menu(void)
{
    int s; 
    
    initcurses();

    while ( (s = selection()) != 3 )
	switch(s) {
	    case 0: // -u
		sousmots();
		break;
	    case 1: // -e et -E
		equiv();
		break;
	    case 2: // -a et -m
		ppsm();
		break;
	    
	}
    
    endwin();
}
